import BaseController from '../common/base.controller'
import { router } from '../decorators/router'
import { ZError } from '../common/ZError'
import { BaseConst } from '../constants/BaseConst'
import { HeroCfg } from '../cfg/parsers/HeroCfg'
import { Hero } from '../models/subdoc/Hero'
import { BagItem } from '../models/BagItem'
import { addHeroDefaultCardGroup } from '../dao/CardGroupDao'
import { ItemInfo } from '../logic/ItemDef'

export default class HeroController extends BaseController {
  @router('post /api/:accountid/heros')
  async herolist(req: any) {
    let account = req.user
    let heros: any[] = []
    for (let [, hero] of account.heros) {
      heros.push(hero.toJson())
    }
    return heros
  }

  @router('post /svr/:accountid/hero/unlock/:heroid')
  async unlockHero(req: any) {
    let account = req.user
    let { heroid, type } = req.params
    if (!heroid) {
      throw new ZError(101, '未指定heroid')
    }
    let hero
    if (!type) {
      hero = await account.unlockHero(heroid, 1)
      // 将该英雄的默认卡组添加到玩家的卡组中,
      // 将默认卡组里的卡添加到玩家可用卡牌中
      let cardMap = account.cardMap
      await addHeroDefaultCardGroup(account.id, hero.heroid, cardMap)
      await account.save()
    } else {
      hero = await account.tryHero(heroid)
      await account.save()

    }
    return hero.toJson()

  }

  @router('post /api/:accountid/hero/update/:heroid')
  async updateHero(req: any) {
    let account = req.user
    let { heroid, items } = req.params
    if (!heroid || !items) {
      throw new ZError(101, '参数不足')
    }
    if (!account.heros.has(heroid + '')) {
      throw new ZError(102, '你未解锁该英雄')
    }
    let hero = account.heros.get(heroid + '')
    if (hero.trial) {
      throw new ZError(103, '不能升级试用英雄')
    }
    let data = await account.updateHero(heroid, items)
    await account.save()
    return data
  }

  /**
   * 解锁英雄卡槽
   * @param req
   * @return {Promise<void>}
   */
  @router('post /api/:accountid/hero/unlockslot/:heroid')
  async unlockCardSlot(req: any) {
    let account = req.user
    let { accountid, heroid } = req.params
    let cfg: HeroCfg = global.$cfg.get(BaseConst.HERO).get(heroid << 0)
    if (!cfg) {
      throw new ZError(10, 'hero not found')
    }
    if (!account.heros.has(heroid + '')) {
      throw new ZError(11, 'hero not unlock')
    }
    let hero: Hero = account.heros.get(heroid + '')
    if (hero.slot >= cfg.groupmax) {
      throw new ZError(12, 'hero slot reach max')
    }
    let needSlot = hero.slot + 1
    // @ts-ignore
    let needItem = cfg[`Unlockgroup${needSlot}id`]
    // @ts-ignore
    let needCount = cfg[`Unlockgroup${needSlot}num`]
    let item = await BagItem.findOne({accountid, itemid: needItem})
    if (!item || item.count < needCount) {
      throw new ZError(13, 'not enough unlock item')
    }
    await BagItem.useItem(accountid, needItem, needCount, 'unlock_cardlot', heroid)
    hero.slot += 1;
    await account.save()
    return {slot: hero.slot}
  }

}
