import BaseController from '../common/base.controller'
import { router } from '../decorators/router'
import { BagItem, getItemType } from '../models/BagItem'
import { ZError } from '../common/ZError'
import { ItemInfo } from '../logic/ItemDef'
import ItemCtrl from '../logic/ItemCtrl'
import { BaseConst } from '../constants/BaseConst'
import { error } from '../common/Debug'

export default class ItemController extends BaseController {
  @router('post /api/:accountid/items')
  async itemList(req: any) {
    let { accountid, type, items } = req.params
    let queryData: any = { accountid, count: {$gt: 0} }
    if (type != undefined) {
      queryData.itemtype = type << 0
    }
    if (items && items.length > 0) {
      queryData.itemid = {$in: items}
    }
    let results = await BagItem.find(queryData)
    return results.map(o => o.toJson())
  }

  @router('post /svr/:accountid/useitem')
  @router('post /api/:accountid/useitem')
  async userItem(req: any) {
    let { accountid, itemid, count } = req.params
    let record = await BagItem.findOne({
      accountid,
      itemid
    })
    if (!record) {
      throw new ZError(11, 'item not found')
    }
    if (record.count < count) {
      throw new ZError(12, 'not enough item')
    }
    await BagItem.useItem(accountid, itemid, count, 'use_item')
    let data: ItemInfo[] = ItemCtrl.useItem(itemid, count)
    await BagItem.addItems(accountid, data, 'use_item', {itemid, count})
    return data
  }

  @router('post /api/:accountid/buyitem')
  async buyItem(req: any) {
    let { itemid, count, accountid } = req.params
    let pinfo = ItemCtrl.getItemPrice(itemid << 0)
    if (pinfo.id == 0) {
      throw new ZError(10, 'item can`t buy')
    }
    let record = await BagItem.findOne({
      accountid,
      itemid: pinfo.id
    })
    if (!record || record.count < pinfo.count * count) {
      throw new ZError(11, 'not enough money')
    }
    await BagItem.useItem(accountid, pinfo.id, pinfo.count * count, 'buy_item')
    let item = new ItemInfo(itemid, count)
    item.type = getItemType(itemid)
    let newItem = await BagItem.addItems(accountid, [item], 'buy_item', pinfo)

    return {
      itemid,
      buynum: count,
      totalnum: newItem.count,
      priceid: pinfo.id,
      pricenum: pinfo.count*count
    }
  }

  @router('post /svr/:accountid/additem')
  async addItem(req: any) {
    let { itemid, count, accountid } = req.params
    let itemInfo = ItemCtrl.findItem(itemid)
    if (!itemInfo) {
      throw new ZError(10, 'no item found')
    }
    let item = new ItemInfo(itemid, count)
    item.type = itemInfo._type
    let results = await BagItem.addItems(accountid, [item], 'svr_add')
    return results[0]
  }

  @router('post /api/:accountid/randomhero')
  async randomHero(req: any) {
    let { accountid } = req.params
    const cfg = global.$cfg.get(BaseConst.ITEMFUNC).get(90003)
    const itemInfos: ItemInfo[] = ItemCtrl.getItemsByInfo(cfg.consume1)
    for (let item of itemInfos) {
      let record = await BagItem.findOne({ accountid, itemid: item.id })
      if (!record || record.count < item.count) {
        throw new ZError(10, 'not enough item' + item.id)
      }
      await BagItem.useItem(accountid, item.id, item.count, 'change_hero')
    }
    return {}
  }

  @router('post /api/:accountid/puzzle')
  async puzzleMap(req: any) {
    let { accountid } = req.params
    const cfg = global.$cfg.get(BaseConst.ITEMFUNC).get(90004)
    const itemInfos: ItemInfo[] = ItemCtrl.getItemsByInfo(cfg.consume1)
    for (let item of itemInfos) {
      let record = await BagItem.findOne({ accountid, itemid: item.id })
      if (!record || record.count < item.count) {
        throw new ZError(10, 'not enough item' + item.id)
      }
    }
    await BagItem.useItems(accountid, itemInfos, 'make_puzzle')
    const resultItems = ItemCtrl.getItemsByInfo(cfg.get)
    if (!resultItems || resultItems.length == 0) {
      error(`拼藏宝图, 未获取到正确的目标物品配置`)
      throw new ZError(11, 'no target item found')
    }
    let newItem = await BagItem.addItems(accountid, resultItems, 'make_puzzle')
    return newItem
  }

}
