import BaseController from '../common/base.controller'
import { router } from '../decorators/router'
import {
  getAccountRank,
  getAccountScore,
  getRankList,
  getRankNear
} from '../service/rank'
import { User } from '../models/User'
const DEFAULT_NICKNAME = '匿名玩家'

export default class RankController extends BaseController {

  @router('post /api/games/rank')
  @router('post /api/:accountid/rank')
  async rank(req: any) {
    let { gameId, channelId, limit, skip, accountid } = req.params
    let start = skip || 0
    let end = start + limit
    let datas: any = await getRankList(start, end)
    let accountIds: string[] = []
    let scoreMap: Map<any, number> = new Map()
    for (let i = 0, l = datas.length; i < l; i += 2) {
      accountIds.push(datas[i])
      scoreMap.set(datas[i], datas[i + 1] << 0)
    }
    const accounts = await User.find({_id : {$in: accountIds}})
    const accountMap: Map<string, any> = accounts.toMap('_id')
    let results: any = []
    let i = 0
    for (let aid of accountIds) {
      const account = accountMap.get(aid) || {nickname: DEFAULT_NICKNAME}
      results.push({
        rank: start + i ++,
        accountid: aid,
        nickname: account.nickname || DEFAULT_NICKNAME,
        avatar: account.avatar,
        score: scoreMap.get(aid)
      })
    }
    let userRank = (await getAccountRank(accountid)) || 999
    // @ts-ignore
    let userScore = (await getAccountScore(accountid)) << 0
    return {
      records: results,
      userRank,
      userScore
    }
  }

  @router('post /api/:accountid/rank/nearMe')
  @router('post /games/rank/nearMe')
  async nearMe(req: any) {
    let { gameId, channelId, score, accountid } = req.params
    let datas = await getRankNear(accountid)
    let accountIds: string[] = []
    let scoreMap: Map<any, any> = new Map()
    for (let data of datas) {
      accountIds.push(data.accountid)
      scoreMap.set(data.accountid, data)
    }

    const accounts = await User.find({_id : {$in: accountIds}})
    const accountMap: Map<string, any> = accounts.toMap('_id')
    let results: any = []
    for (let aid of accountIds) {
      const account = accountMap.get(aid)
      results.push({
        rank: scoreMap.get(aid).rank,
        accountid: aid,
        nickname: account.nickname || DEFAULT_NICKNAME,
        avatar: account.avatar,
        score: scoreMap.get(aid).score
      })
    }
    return results
  }
}
