import { User } from '../models/User'
import { ZError } from '../common/ZError'
import { BaseConst } from '../constants/BaseConst'
import { updateRank } from '../service/rank'
import { Card } from '../models/subdoc/Card'
import { Hero } from '../models/subdoc/Hero'
import { addHeroDefaultCardGroup } from './CardGroupDao'
import { ItemInfo } from '../logic/ItemDef'
import ItemCtrl from '../logic/ItemCtrl'
import { BagItem } from '../models/BagItem'

export async function fetchAccount({ accountid, nickname, avatar, robot = false }
                                   : { accountid: string, nickname?: string, avatar?: string, robot?: boolean }) {
  let account = (await User.findOrCreate({ _id: accountid })).doc
  if (account.locked) {
    throw new ZError(4, 'account locked')
  }
  const formulaCfg = global.$cfg.get(BaseConst.FORMULA)
  if (account.season_score == -1) {
    account.season_score = formulaCfg.get(70003).number
    await updateRank(accountid, account.season_score)
  }
  if (robot) {
    account.robot = 1
  }
  if (nickname) {
    account.nickname = nickname
  }
  if (avatar) {
    account.avatar = avatar
  }
  if (!robot) {
    let baseCfg = global.$cfg.get(BaseConst.COMPOUND)
    let initItem = baseCfg.get(BaseConst.PLAYER_INIT_ITEMS)
    if (initItem) {
      const initItemStr = `${initItem.value}:1`
      const itemInfos: ItemInfo[] = ItemCtrl.getItemsByInfo(initItemStr)
      await BagItem.addItems(account._id, itemInfos, 'init_item')
    }
  }
  let cardMap = account.cardMap
  for (let [, cfg] of global.$cfg.get(BaseConst.EFFECTCARD)) {
    if (cfg.org_gift == 1 && cfg.type_id == 1 && !cardMap.has(cfg.id + '')) {
      const card = new Card()
      card.cardid = cfg.id
      card.owned = true
      card.ban = false
      card.usetype = 0
      card.free = true
      card.free_expire = 0
      card.time = Date.now()
      cardMap.set(card.cardid + '', card)
    }
  }
  for (let [, cfg] of global.$cfg.get(BaseConst.HERO)) {
    if (cfg.org_gift == 1) {
      let hero = new Hero()
      hero.heroid = cfg.id
      hero.free = true
      hero.trial = false
      hero.level = 1
      hero.slot = 1
      hero.time = Date.now()
      hero.exp = 0
      if (!account.heros.has(cfg.id + '')) {
        account.heros.set(cfg.id + '', hero)
        // await addHeroDefaultCardGroup(accountid, hero.heroid, cardMap)
      }
    }
  }

  await account.save()
  return account
}
