import { BaseConst } from '../constants/BaseConst'
import { CardGroup } from '../models/CardGroup'
import { Card } from '../models/subdoc/Card'

export async function addHeroDefaultCardGroup(accountid: string, heroid: number, cardMap: Map<string, Card>) {
  const heroCfg = global.$cfg.get(BaseConst.HERO).get(heroid)
  let cardgroup = new CardGroup({})
  let cards: Card[] = []
  for (let i = 1; i <= 4; i++) {
    const cardid = heroCfg[`follower${ i }id`]
    const owned = cardMap.has(cardid)
    const card = new Card()
    card.cardid = cardid
    card.owned = owned
    card.ban = false
    card.usetype = 0
    card.free = !owned
    card.free_expire = 0
    card.time = Date.now()
    cards.push(card)
  }
  cardgroup.accountid = accountid
  cardgroup.heroid = heroCfg.id
  cardgroup.selected = true
  cardgroup.isdefault = false
  cardgroup.cards = cards
  await cardgroup.save()
}

/**
 * 获取英雄默认卡组
 * @param {number} heroid
 * @return {{gid: string, cards: {free_expire: number, owned: boolean, cardid: number, count: number, time: number, free: boolean, usetype: number, ban: boolean}[], heroid: number, isdefault: boolean, selected: boolean}}
 */
export function getDefaultCardGroupByHero(heroid: number | string) {
  heroid = +heroid
  const heroCfg = global.$cfg.get(BaseConst.HERO).get(heroid)
  let cardgroup = new CardGroup({})
  let cards: Card[] = []
  for (let i = 1; i <= 4; i++) {
    const cardid = heroCfg[`follower${ i }id`]
    const card = new Card()
    card.cardid = cardid
    card.owned = true
    card.ban = false
    card.usetype = 0
    card.free = true
    card.free_expire = 0
    card.time = Date.now()
    cards.push(card)
  }
  cardgroup.heroid = heroCfg.id
  cardgroup.selected = false
  cardgroup.isdefault = true
  cardgroup.cards = cards
  const data = cardgroup.toJson()
  data.gid = 'default'
  return data
}
