import { BaseConst } from '../constants/BaseConst'
import { updateRank } from '../service/rank'
import { RecordInfo } from '../models/User'
import { timeBeforeDay } from '../utils/time.util'
import { GamePlayer } from '../models/GameRecord'


export async function parsePlayerMatchDate(player: GamePlayer, user: any, winner: number) {
  const fc = global.$cfg.get(BaseConst.FORMULA)
  const scores = [
    fc.get(70043).number,
    fc.get(70046).number,
    fc.get(70044).number,
    fc.get(70045).number,
    fc.get(70047).number,
    fc.get(70048).number
  ]
  if (!player.escape) {
    user.season_score = Math.max((user.season_score + player.scoreChange) | 0, fc.get(70002).number)
    await updateRank(user._id, user.season_score)
  }

  if (!user.season_data) {
    user.season_data = new Map()
  }
  /**
   * begin of 将赛季排位记录放入season_records, 方便计算10胜
   */
  if (!player.escape) {
    let data = new RecordInfo()
    data.time = Date.now()
    if (player.team == winner) {
      data.status = 2
    } else if (winner == -1) {
      data.status = 1
    } else {
      data.status = 0
    }
    user.season_records.push(data)

    let records = user.season_records
    while (records.length > 10) {
      records.pop()
    }
    let smallTime = timeBeforeDay(30)
    for (let r of records) {
      if (r.time < smallTime) {
        records.removeEx(r)
      }
    }
  }

  /**
   * end of 将赛季排位记录放入season_records, 方便计算10胜
   */

  /**
   * begin of 处理比赛统计信息
   */
  if (!player.escape) {
    if (winner == player.team) {
      user.season_data.zinc('win', 1)
    } else {
      user.season_data.zinc('loss', 1)
    }
    let statData = player.statdata
    let honorp = 0
    for (let key in statData) {
      let numKey = +key

      if (numKey >= scores.length) {
        continue
      }
      // @ts-ignore
      let val = Number(statData[key])
      honorp += val * scores[numKey]
      if (numKey == 4 || numKey == 5) {
        user.season_data.zinc(key + '_total', val)
      } else {
        user.season_data.zinc(key + '_total', val * scores[numKey])
      }

      if (user.season_data.has(key + '')) {
        user.season_data.set(key + '', Math.max(user.season_data.get(key + ''), val))
      } else {
        user.season_data.set(key + '', val)
      }
    }
    user.season_data.zinc('honor_ap', honorp)
    user.season_data.zinc('mvp_ap', player.mvpscore)
  }
  /**
   * end of 处理比赛统计信息
   */
}
