import { dbconn } from '../decorators/dbconn'
import {
  getModelForClass,
  index,
  modelOptions,
  mongoose,
  prop,
  Severity
} from '@typegoose/typegoose'
import { Base } from '@typegoose/typegoose/lib/defaultClasses'
import { ItemInfo } from '../logic/ItemDef'
import { BagItem } from './BagItem'


@dbconn()
@index({'accountid': 1, 'itemid': 1}, {unique: false})
@index({'accountid': 1 }, {unique: false})
@modelOptions({
  schemaOptions:
    {collection: "item_record", timestamps: true},
  options: {allowMixed: Severity.ALLOW}
})
class ItemRecordClass extends Base<string>{
  @prop()
  public itemid: number;
  @prop()
  public accountid: string;
  @prop()
  public count: number;
  @prop()
  public reason: string;
  @prop({type: mongoose.Schema.Types.Mixed})
  public data: {};

  public static async log(accountid: string, itemid: number, count: number, reason?: string, moreparam?: any) {
    let record = new ItemRecord()
    record.accountid = accountid
    record.itemid = itemid
    record.count = count
    record.reason = reason
    record.data = moreparam
    await record.save()
  }
}


export const ItemRecord = getModelForClass(ItemRecordClass,
  // @ts-ignore
  {existingConnection: ItemRecordClass['db']})

