import { singleton } from '../decorators/singleton.decorator'
import { allSvrs, closeRoom } from '../service/WsSvr'
import * as schedule from 'node-schedule'
import logger from '../logger/logger'

@singleton
export class SvrStatSchedule {
  async parseAll() {
    let { data } = await allSvrs()
    logger.info(`check room state, rooms: ${data.rooms.length} ,connections: ${data.connections}, cpu: ${data.cpu}, memory: ${data.memory.usedMemMb} / ${data.memory.totalMemMb}`)
    for (let room of data.rooms) {
      if (room.name === 'general_room' && room.clients < 4 && room.elapsedTime >= 600000) { // 10 * 60 * 1000
        logger.info(`room ${room.roomId} idle`)
        try {
          await closeRoom(room.roomId)
        } catch {
          logger.error(`room ${room.roomId} close error`)
        }
      }
    }
  }

  scheduleAll() {
    const job = schedule.scheduleJob('1 * * * * *', async () => {
      try {
        await this.parseAll()
      } catch (err) {
        logger.error(err)
      }
    })
  }
}
