import axios from 'axios'
import { createSign } from '../utils/security.util'
import { RedisClient } from '../redis/RedisClient'
import { BaseConst } from '../constants/BaseConst'

const CONFIG_URL = 'https://center.kingsome.cn/api/cfg_list'
/**
 * 获取jcfw中该游戏的配置
 * @param {string} gameid
 * @param {string} channel
 */
export function getGameConfig(gameid: string, channel: string) {
  let url = `${ CONFIG_URL }?game_id=${ gameid }&channel_id=${ channel }`
  return axios.get(url).then((res: any) => {
    if (res.data.errorcode || !res.data.result) {
      throw new Error(`error get game cfg, code: ${ res.errorcode }, msg: ${ res.errmsg }`)
    }
    return JSON.parse(res.data.result)
  })
}

/**
 * 获取随机数量的机器人数据
 * @param {string | undefined} accountId
 * @param {number} num
 * @return {Promise<AxiosResponse<any>>}
 */
export function randomUser(accountId: string | undefined, num: number) {
  const excludeAccountids = accountId ? `[${ accountId }]` : '[]'
  const timestamp = new Date().getTime()
  const url = 'https://service.kingsome.cn/webapp/index.php?c=Voodoo&a=getRobotList'
  const paramStr = `exclude_accountids=${ excludeAccountids }&exclude_names=[]&num=${ num }`
  const signStr = createSign('70e32abc60367adccaa9eb7b56ed821b', paramStr, timestamp)
  const link = `${ url }&${ paramStr }&sign=${ signStr }&timestamp=${ timestamp }`
  return axios.get(link).then((res: any) => {
    if (res.data.errorcode || !res.data.robot_list) {
      throw new Error(`error get robo list, code: ${ res.errorcode }, msg: ${ res.errmsg }`)
    }
    return res.data.robot_list
  })
}

/**
 * 更新redis中机器人数据
 * @param {number} count
 * @param {boolean} force
 */
export async function initRobotCache(count: number, force?: boolean) {
  const redisClient = new RedisClient()
  const countExists = await redisClient.scard(BaseConst.ROBOT_INFO)
  if (countExists > count && !force) {
    return
  }
  let robots = await randomUser('', 100)
  if (robots && Array.isArray(robots)) {
    for(const robot of robots) {
      await redisClient.sadd(BaseConst.ROBOT_INFO, `${robot.nickname}${BaseConst.ROBOT_INFO_SEP}${robot.avatar_url}`)
    }
  }
}
